%
%   BW_Stable_Period.M   This file provides a test case of the Stable Period
%                            Method for determining Blade Worth Curves
%
%   The basic test scenario is as follows:
%     1.  With a given blade worth curve, we can simulate P(t) for several dz (note
%         that some of these are a little larger than would be used in reality).
%     2.  With the P(t) for each perturbation, we use the Stable Period or Doubling 
%         Time method to calculate drho for each dz.  
%     3.  With the simulated drho/dz data, we can do a curve fit to find the 
%         differential worth curve and then integrate this to get the integral 
%         worth curve.  This uses the same basic logic as the blade_worth GUI 
%         program, but Matlab's built-in curve fitting routines are used...
%     4.  The test: If the computed integral worth curve is close to the initially 
%         assumed profile, then this should be a good test of the basic theory and 
%         actual implementation of the method.
%
%   This simulation uses Blade 3 as the BOI with blade worth data from Jan. 2018...
%
%   Let's give this a try...
%   
%   File prepared J. R. White, UMass-Lowell (last update:  March 2018)
%

      clear all,  close all,  nfig = 0;  
%%
%   define UMLRR-specific kinetics data and other parameters for the simulations
      [Be,B,Gt,lam,td,S,kappa,nu] = kinetics_data();
      tol  = 1e-5;  options = odeset('RelTol',tol); % error tolerance for ODE solver
      ftime = 300;            % final simulation time (seconds)
      dt = 1.0;               % sampling time (sec)
      t = (0:dt:ftime)';      % time vector with sampling time set by dt
      nlevel = 0.05;          % random noise level (random uncertainty in P(t) data)
%
      rho0 = 0;  Q = 0;       % critcal initial state  (i.e. rho = 0 and no source) 
      Po = 1;                 % initial power level (magnitude not important here)
      Co = B*Po./(lam*Gt);    % initial precursor conc
      xo = [Po Co]';          % initial condition for state vector
      boi = 3;  yr = 'Jan2018';    % blade of interest (Blade 3) & year of worth data 
      [dum,H] = bworth(0,boi,yr);  % simply returns the max height for the BOI
      rate = 3.7/60;          % rate of blade movement for BOI (inches/sec)
      zlocs = [0 4 8 10 12 13 14 16 18 22 H];  % blade locations (inches withdrawn) 
%%
%   do simulation for each dz (must be piecewise linear)
      Ndz = length(zlocs)-1;   dz = zeros(Ndz,1);
      z1 = dz;  z2 = dz;  zmid = dz;  drho = dz;  DT = dz;  tau = dz;  % initialize
      for i = 1:Ndz
        z1(i) = zlocs(i);   z2(i) = zlocs(i+1);
        dz(i) = zlocs(i+1)-zlocs(i);
        zmid(i) = (zlocs(i+1)+zlocs(i))/2;
        ftx = @(t,x) pkeqns_rampb_nofdbk(t,x,rho0,Q,Be,Gt,B,lam,kappa,nu, ...
                                         rate,zlocs(i:i+1),boi,yr);
        [t,x] = ode15s(ftx,t,xo,options);
        P = x(:,1)/Po;          % normalized P(t)
%   add some noise to the P(t) signal
        rn = rand(size(t));     % vector of uniform random numbers (0 to 1)
        a = 1-nlevel;   b = 1+nlevel;  % nlevel is the 'noise level'  (see above)
        srn = (b-a)*rn + a;     % uniformly distributed between a and b
        srn(1) = 1.0;           % reset initial value to 1.0
        Pn = srn.*P;            % P(t) with random noise
%   compute log(P), extract slope of curve after some transient time, and calc rho
        Pln = log(Pn);   tt = 90;                           
        Plnfit = Pln(t > tt);   tfit = t(t > tt);  
        cc = polyfit(tfit,Plnfit,1);   slope = cc(1);   
        meas_period = 1/slope;  meas_dt = meas_period*log(2); 
        meas_rho = sum(B./(1 + lam*meas_period));
        drho(i) = meas_rho;  DT(i) = meas_dt;  tau(i) = meas_period;
      end
      drho = drho*100;   dkkpi = drho./dz;  % %Dk/k  and  %Dk/k per inch
%% 
%   at this point, one needs the following vectors to be available for the curve fit:
%      z1, z2, and zmid -- these represent the start, end, and midpoint values for
%                          the actual blade locations
%      DT -- this is the measured doubling time
%      dkkpi -- this is the %Dk/k per inch differential worth as derived from the 
%               reactivity eqn (given DT, calc period, use reeactivity eqn. to get
%               Dk/k, and form %Dk/k per inch by dividing by dz for step
%
%   now add on a zero-worth point at the lower end (similar to blade_worth GUI)
      z1 = [0; z1 ];    z2 = [0; z2 ];      DT = [0; DT ];
      x = [0; zmid ];   y = [0; dkkpi ];    N = length(x);
%
%   edit raw data and the computed data for the differential worth curve fits 
      fprintf(1,'\n  Raw Simulated + Computed Data \n');
      fprintf(1,'    Initial Ht.     Final Ht.   Doubling Time    Midpoint    Diff. Worth  \n');
      fprintf(1,'       (in)           (in)          (sec)          (in)     (%%Dk/k per in) \n');
      for n = 1:N
        fprintf(1,'   %8.2f       %8.2f      %8.1f       %8.2f      %8.4f \n', ...
                 z1(n),z2(n),DT(n),x(n),y(n));
      end
%
%   use Matlab's built-in fitting routines to do the curve fit for the differential 
%   worth to the Combined Polynomial and Sinusoid Model (PSM for short)
%    math model:  drho(z)/dz = c1 + c2*z + c3*z^2 + c4*z^3 + c5*cos(2*pi*z/H)
      expr = {'1','x','x.^2','x.^3','cos(2*pi*x)'}; % x is normalized height (i.e. z/H)
      PSM = fittype(expr);                          % setup for Matlab's 'fit' command
      options = fitoptions('Method','LinearLeastSquares','Normalize','off');
      [cfun,good] = fit(x/H,y,PSM,options);         % do actual curve fit 
      c = coeffvalues(cfun);                        % normalized coeffs of math model 
      r2 = good.rsquare;                            % coeff of determination (i.e. r^2)
      c(2) = c(2)/H;  c(3) = c(3)/H^2;  c(4) = c(4)/H^3;  % coeffs for regular model
%      
%   now plot data for differential worth (both models and the simulated expt. data)
      NN = 100;    xx = linspace(x(1),H,NN);
      yy = polyval(c(4:-1:1),xx) + c(5)*cos(2*pi*xx/H);  
      nfig = nfig+1;   figure(nfig)
      plot(xx,yy,'r-',x,y,'bo','LineWidth',2);grid;
      set(gca,'XTick',0:2:H);
      v = axis;  v(2) = H;  axis(v);
      title(['Stable\_Period: Simulated Differential Worth Curve for Blade #',num2str(boi)])
      xlabel('Distance Withdrawn (inches)')
      ylabel('Differential Worth (%\DeltaK/K per inch)')
      legend('Combined Sine+Poly Model','Simulated Points','Location','South')
%
%   calc and plot data for integral worth
      ci = [c(4)/4 c(3)/3 c(2)/2 c(1) 0];
      yyi = polyval(ci,xx) + c(5)*(H/(2*pi))*sin(2*pi*xx/H);  
      nfig = nfig+1;   figure(nfig)
      rho_act = bworth(xx,boi,yr)-bworth(xx(1),boi,yr);
      plot(xx,yyi,'r-',xx,rho_act,'b--','LineWidth',2);grid;
      set(gca,'XTick',0:2:H);
      v = axis;  v(2) = H;  v(3) = 0;  axis(v);
      title(['Stable\_Period: Integral Worth Test for Blade #',num2str(boi)])
      xlabel('Distance Withdrawn (inches)'),ylabel('Integral Worth (%\DeltaK/K)')
      legend(['Simulated \rho_{tot} = ',num2str(yyi(NN),'%5.2f'),'%\DeltaK/K'], ...
             ['Actual \rho_{tot} = ',num2str(rho_act(NN),'%5.2f'),'%\DeltaK/K'], ...
              'Location','NorthWest')
%
%   summary edits
      fprintf(1,'\n  Coeffs for Combined Poly-Sinusoid Differential Worth Model:\n    %12.4e %12.4e %12.4e %12.4e %12.4e \n',c);
      fprintf(1,'\n  Curve fit coeff of determination (r-squared): %10.4f\n',r2);
      fprintf(1,'  Total worth based on curve fit (%%Dk/k):       %10.4f\n',yyi(NN));
      fprintf(1,'  Actual total worth of Blade %i (%%Dk/k):        %10.4f\n',boi,rho_act(NN));
      fprintf(1,'\n');
%
%   end of program
